/************************************************************************/
/*                                                                      */
/* Copyright (C) 2004 Mattias Eklf <tias@ludd.ltu.se>                  */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with this program; if not, write to the Free Software          */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA            */
/* 02111-1307, USA.                                                     */
/*                                                                      */
/************************************************************************/

#include <gkrellm2/gkrellm.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "online.xpm"
#include "away.xpm"
#include "na.xpm"
#include "occ.xpm"
#include "dnd.xpm"
#include "ffc.xpm"
#include "offline.xpm"
#include "invisible.xpm"

#include "message.xpm"
#include "file.xpm"
#include "url.xpm"
#include "sms.xpm"
#include "chat.xpm"

#define PLUGIN_PLACEMENT	MON_MAIL
#define THEME_NAME "licq"

#define STATUS_FILE_NAME "/.licq/infofs/status"
#define PENDING_FILE_NAME "/.licq/infofs/pending"
#define NUM_EVENTS_FILE_NAME "/.licq/infofs/num_events"

static GkrellmMonitor  *monitor;
static GkrellmPanel    *panel;
static GkrellmTicks    *pGK;
static gint style_id;

static GkrellmPiximage *image_online;
static GkrellmPiximage *image_away;
static GkrellmPiximage *image_na;
static GkrellmPiximage *image_occ;
static GkrellmPiximage *image_dnd;
static GkrellmPiximage *image_ffc;
static GkrellmPiximage *image_offline;
static GkrellmPiximage *image_invisible;

static GkrellmPiximage *image_message;
static GkrellmPiximage *image_file;
static GkrellmPiximage *image_url;
static GkrellmPiximage *image_sms;
static GkrellmPiximage *image_chat;

static GkrellmDecal *decal_online;
static GkrellmDecal *decal_away;
static GkrellmDecal *decal_na;
static GkrellmDecal *decal_occ;
static GkrellmDecal *decal_dnd;
static GkrellmDecal *decal_ffc;
static GkrellmDecal *decal_offline;
static GkrellmDecal *decal_invisible;

static GkrellmDecal *decal_message;
static GkrellmDecal *decal_file;
static GkrellmDecal *decal_url;
static GkrellmDecal *decal_sms;
static GkrellmDecal *decal_chat;

static GkrellmDecal *decal_num_events;

static GkrellmDecal *vis_decal_status = NULL;
static GkrellmDecal *vis_decal_pending = NULL;
	

static char status_file_abs_path[256];
static char pending_file_abs_path[256];
static char num_events_file_abs_path[256];

static void update_plugin()
{
	int fd;
	char buf[64];
	int num_events;

	if ((pGK->timer_ticks % 2) == 0) {
		fd = open(status_file_abs_path, O_RDONLY);
		read(fd, buf, sizeof(buf));
		close(fd);

		/* Make current status decal invisible. */
		if (vis_decal_status != NULL)
			gkrellm_make_decal_invisible(panel, vis_decal_status);

		/* Check what's the current status and make corresponding decal visible. */
		if (strncmp(buf, "On", 2*sizeof(char)) == 0)
			vis_decal_status = decal_online;
		else if (strncmp(buf, "Away", 4*sizeof(char)) == 0)
			vis_decal_status = decal_away;
		else if (strncmp(buf, "N/A", 3*sizeof(char)) == 0)
			vis_decal_status = decal_na;
		else if (strncmp(buf, "Occ", 3*sizeof(char)) == 0)
			vis_decal_status = decal_occ;
		else if (strncmp(buf, "DND", 3*sizeof(char)) == 0)
			vis_decal_status = decal_dnd;
		else if (strncmp(buf, "FFC", 3*sizeof(char)) == 0)
			vis_decal_status = decal_ffc;
		else if (strncmp(buf, "Off", 3*sizeof(char)) == 0)
			vis_decal_status = decal_offline;
		else if (strncmp(buf, "(", 1*sizeof(char)) == 0)
			vis_decal_status = decal_invisible;
		else
			vis_decal_status = NULL;

		if (vis_decal_status != NULL)
			gkrellm_make_decal_visible(panel, vis_decal_status);
		
		fd = open(num_events_file_abs_path, O_RDONLY);
		read(fd, buf, sizeof(buf)-1);
		close(fd);
		buf[sizeof(buf)-1] = '\0';

		/* Clear the num events text decal, because it is about to be updated. */
		gkrellm_decal_text_clear(decal_num_events);

		/* Check if there's any event pending. */
		num_events = atoi(buf);
		if (num_events > 0) {
			sprintf(buf, "%i", num_events);

			/* Update num events text decal. */
			gkrellm_decal_text_insert(decal_num_events, buf, gkrellm_meter_textstyle(style_id), 0, 0);


			fd = open(pending_file_abs_path, O_RDONLY);
			read(fd, buf, sizeof(buf)-1);
			close(fd);

			/* Check what's the first pending event and make it visible. */
			if (strncmp(buf, "FILE", 4*sizeof(char)) == 0) {
				if (vis_decal_pending != decal_file && vis_decal_pending != NULL)
					gkrellm_make_decal_invisible(panel, vis_decal_pending);
				vis_decal_pending = decal_file;
			} else if (strncmp(buf, "URL", 3*sizeof(char)) == 0) {
				if (vis_decal_pending != decal_url && vis_decal_pending != NULL)
					gkrellm_make_decal_invisible(panel, vis_decal_pending);
				vis_decal_pending = decal_url;
			} else if (strncmp(buf, "SMS", 3*sizeof(char)) == 0) {
				if (vis_decal_pending != decal_sms && vis_decal_pending != NULL)
					gkrellm_make_decal_invisible(panel, vis_decal_pending);
				vis_decal_pending = decal_sms;
			} else if (strncmp(buf, "CHAT", 4*sizeof(char)) == 0) {
				if (vis_decal_pending != decal_chat && vis_decal_pending != NULL)
					gkrellm_make_decal_invisible(panel, vis_decal_pending);
				vis_decal_pending = decal_chat;
			} else {
				if (vis_decal_pending != decal_message && vis_decal_pending != NULL)
					gkrellm_make_decal_invisible(panel, vis_decal_pending);
				vis_decal_pending = decal_message;
			}
		} else {
			/* No events pending, so make the pending event decal invisible. */
			gkrellm_make_decal_invisible(panel, vis_decal_pending);			
			vis_decal_pending = NULL;

			/* No events pending, so make the num events text decal an empty string. */
			gkrellm_decal_text_insert(decal_num_events, "", gkrellm_meter_textstyle(style_id), 0, 0);
		}
	}

	/* We want the pending event decal to be blinking. */
	if ((pGK->timer_ticks % 8) == 0 && vis_decal_pending != NULL) {
		if (gkrellm_is_decal_visible(vis_decal_pending))
			gkrellm_make_decal_invisible(panel, vis_decal_pending);
		else
			gkrellm_make_decal_visible(panel, vis_decal_pending);
	}

	/* Update panel. */
       	gkrellm_draw_panel_layers(panel);
}

static gint panel_expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
	gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			panel->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
			ev->area.width, ev->area.height);	
	
	return FALSE;
}

static void create_plugin(GtkWidget *vbox, gint first_create)
{
	char *home;
	GkrellmStyle     *style;
	GkrellmTextstyle *ts;

	home = getenv("HOME");

	/* OOPS! Possible source of overflow! */
	strcpy(status_file_abs_path, home);
	strcat(status_file_abs_path, STATUS_FILE_NAME);
	strcpy(pending_file_abs_path, home);
	strcat(pending_file_abs_path, PENDING_FILE_NAME);
	strcpy(num_events_file_abs_path, home);
	strcat(num_events_file_abs_path, NUM_EVENTS_FILE_NAME);

	if (first_create)
		panel = gkrellm_panel_new0();

	vis_decal_status = NULL;
	vis_decal_pending = NULL;
	
	style = gkrellm_meter_style(style_id);

	/* Load the status piximages. */
	gkrellm_load_piximage("online", online_xpm, &image_online, THEME_NAME);
	gkrellm_load_piximage("away", away_xpm, &image_away, THEME_NAME);
	gkrellm_load_piximage("na", na_xpm, &image_na, THEME_NAME);
	gkrellm_load_piximage("occ", occ_xpm, &image_occ, THEME_NAME);
	gkrellm_load_piximage("dnd", dnd_xpm, &image_dnd, THEME_NAME);
	gkrellm_load_piximage("ffc", ffc_xpm, &image_ffc, THEME_NAME);
	gkrellm_load_piximage("offline", offline_xpm, &image_offline, THEME_NAME);
	gkrellm_load_piximage("invisible", invisible_xpm, &image_invisible, THEME_NAME);						

	/* Load the event piximages. */
	gkrellm_load_piximage("message", message_xpm, &image_message, THEME_NAME);
	gkrellm_load_piximage("file", file_xpm, &image_file, THEME_NAME);
	gkrellm_load_piximage("url", url_xpm, &image_url, THEME_NAME);
	gkrellm_load_piximage("sms", sms_xpm, &image_sms, THEME_NAME);
	gkrellm_load_piximage("chat", chat_xpm, &image_chat, THEME_NAME);

	/* Make decals out of the status piximages, and place them. */
	decal_online = gkrellm_make_scaled_decal_pixmap(panel, image_online, style, 1, 4, 2, 0, 0);
	decal_away = gkrellm_make_scaled_decal_pixmap(panel, image_away, style, 1, 4, 2, 0, 0);
	decal_na = gkrellm_make_scaled_decal_pixmap(panel, image_na, style, 1, 2, 4, 0, 0);
	decal_occ = gkrellm_make_scaled_decal_pixmap(panel, image_occ, style, 1, 4, 2, 0, 0);
	decal_dnd = gkrellm_make_scaled_decal_pixmap(panel, image_dnd, style, 1, 4, 2, 0, 0);
	decal_ffc = gkrellm_make_scaled_decal_pixmap(panel, image_ffc, style, 1, 4, 2, 0, 0);
	decal_offline = gkrellm_make_scaled_decal_pixmap(panel, image_offline, style, 1, 4, 2, 0, 0);
	decal_invisible = gkrellm_make_scaled_decal_pixmap(panel, image_invisible, style, 1, 4, 2, 0, 0);

	/* Make decals out of the event piximages, and place them. */
	decal_message = gkrellm_make_scaled_decal_pixmap(panel, image_message, style, 1, 6+decal_online->w, 2, 0, 0);
	decal_file = gkrellm_make_scaled_decal_pixmap(panel, image_file, style, 1, 6+decal_online->w, 2, 0, 0);
	decal_url = gkrellm_make_scaled_decal_pixmap(panel, image_url, style, 1, 6+decal_online->w, 2, 0, 0);
	decal_sms = gkrellm_make_scaled_decal_pixmap(panel, image_sms, style, 1, 6+decal_online->w, 2, 0, 0);
	decal_chat = gkrellm_make_scaled_decal_pixmap(panel, image_chat, style, 1, 6+decal_online->w, 2, 0, 0);

	/* Make a text decal for num events, and place it. */
	ts = gkrellm_meter_textstyle(style_id);
	decal_num_events = gkrellm_create_decal_text_with_height(panel, ts, style, 10+decal_online->w+decal_message->w, 4, 16, 16, 10);

	/* Create the panel. */
        gkrellm_panel_configure(panel, NULL, style);
        gkrellm_panel_create(vbox, monitor, panel);

	/* Draw the status decals onto the panel. */
	gkrellm_draw_decal_pixmap(panel, decal_online, 0);
	gkrellm_draw_decal_pixmap(panel, decal_away, 0);
	gkrellm_draw_decal_pixmap(panel, decal_na, 0);
	gkrellm_draw_decal_pixmap(panel, decal_occ, 0);
	gkrellm_draw_decal_pixmap(panel, decal_dnd, 0);
	gkrellm_draw_decal_pixmap(panel, decal_ffc, 0);
	gkrellm_draw_decal_pixmap(panel, decal_offline, 0);
	gkrellm_draw_decal_pixmap(panel, decal_invisible, 0);

	/* Draw the event decals onto the panel. */
	gkrellm_draw_decal_pixmap(panel, decal_message, 0);
	gkrellm_draw_decal_pixmap(panel, decal_file, 0);
	gkrellm_draw_decal_pixmap(panel, decal_url, 0);
	gkrellm_draw_decal_pixmap(panel, decal_sms, 0);
	gkrellm_draw_decal_pixmap(panel, decal_chat, 0);

	/* Clear the num events text decal to begin with. */
	gkrellm_decal_text_clear(decal_num_events);

	/* Make all status decals invisble to begin with. */
	gkrellm_make_decal_invisible(panel, decal_online);
	gkrellm_make_decal_invisible(panel, decal_away);
	gkrellm_make_decal_invisible(panel, decal_na);
	gkrellm_make_decal_invisible(panel, decal_occ);
	gkrellm_make_decal_invisible(panel, decal_dnd);
	gkrellm_make_decal_invisible(panel, decal_ffc);
	gkrellm_make_decal_invisible(panel, decal_offline);
       	gkrellm_make_decal_invisible(panel, decal_invisible);

	/* Make all event decals invisible to begin with. */
	gkrellm_make_decal_invisible(panel, decal_message);
	gkrellm_make_decal_invisible(panel, decal_file);
	gkrellm_make_decal_invisible(panel, decal_url);
	gkrellm_make_decal_invisible(panel, decal_sms);
	gkrellm_make_decal_invisible(panel, decal_chat);
	
	if (first_create) {
		g_signal_connect(G_OBJECT (panel->drawing_area), "expose_event",
				 G_CALLBACK(panel_expose_event), NULL);
	}
}

static GkrellmMonitor  plugin_mon  =
	{
		"Licq",           /* Name, for config tab.        */
		0,                /* Id,  0 if a plugin           */
		create_plugin,    /* The create_plugin() function */
		update_plugin,    /* The update_plugin() function */
		NULL,             /* The create_plugin_tab() config function */
		NULL,             /* The apply_plugin_config() function      */
		
		NULL,             /* The save_plugin_config() function  */
		NULL,             /* The load_plugin_config() function  */
		"Licq",           /* config keyword                     */
		
		NULL,             /* Undefined 2  */
		NULL,             /* Undefined 1  */
		NULL,             /* Undefined 0  */
		
		PLUGIN_PLACEMENT, /* Insert plugin before this monitor.       */
		NULL,             /* Handle if a plugin, filled in by GKrellM */
		NULL              /* path if a plugin, filled in by GKrellM   */
	};

GkrellmMonitor *gkrellm_init_plugin(void)
{
	pGK = gkrellm_ticks();
	
	style_id = gkrellm_add_meter_style(&plugin_mon, THEME_NAME);
	monitor = &plugin_mon;

	return &plugin_mon;
}
